<?php
// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

/**
 * Admin Class
 *
 * Manage Admin Panel Class
 *
 * @package Askcybex AI - Blog Generation
 * @since 1.0.0
 */
class Aibg_Admin {

	public $scripts;

	//class constructor
	function __construct() {

		global $aibg_scripts;

		$this->scripts = $aibg_scripts;
	}

	function aibg_save_api_key(){
		
		if ( !isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'nonce') ) 
		{
			wp_send_json_error('Invalid nonce');
		}

		$aibg_api_key = $_POST['aibg_api_key'];

		update_option( 'aibg_api_key', $aibg_api_key );

		echo wp_json_encode( array( 'success' => true ) );
		exit;
	}

	function aibg_generate_title() {
	
		if ( !isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'nonce') ) 
		{
			wp_send_json_error('Invalid nonce');
		}

		$aibg_words 		= $_POST['aibg_words'];

		$aibg_generative 	= trim($_POST['aibg_generative']);

		$aibg_api_key 		= $_POST['aibg_api_key'];
	
		$aibg_prompt_key 	= $aibg_generative != 'false' ? 'prompt' : 'msg';
		
		$aibg_prompt 		= 'As a blogger, generate a blog title about - ' . $aibg_words;
	
		$aibg_api_url 		= 'https://askcybex.com/api/chat';
	
		$aibg_api_data 		= array(
			'api_key' 			=> $aibg_api_key,
			$aibg_prompt_key 	=> $aibg_prompt
		);
	
		$response 		= wp_remote_post( $aibg_api_url, array(
			'method' 	=> 'POST',
			'body' 		=> $aibg_api_data,
			'timeout' 	=> 45,
			'headers' 	=> array()
		));
	
		if ( is_wp_error($response) ) 
		{
			$error_message = $response->get_error_message();

			echo wp_json_encode(array(
				'success' 	=> false,
				'error' 	=> $error_message
			));

		} 
		else 
		{
			$body 				= wp_remote_retrieve_body($response);

			$aibg_api_response 	= json_decode($body);
	
			if ( isset($aibg_api_response->response) ) 
			{
				$aibg_response_title = $aibg_api_response->response;
	
				echo wp_json_encode(array(
					'success' 		=> true,
					'aibg_title' 	=> $aibg_response_title
				));
			} 
			else 
			{
				echo wp_json_encode(array(
					'success' 	=> false,
					'error' 	=> isset($aibg_api_response->error) ? $aibg_api_response->error : 'Unknown error'
				));
			}
		}
		exit;
	}
	
	function aibg_generate_single_description() {

		if ( !isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'nonce') ) 
		{
			wp_send_json_error('Invalid nonce');
		}

		$aibg_data_index 	= $_POST['aibg_data_index'];

		$post_title 		= $_POST['post_title'];

		$aibg_generative 	= trim($_POST['aibg_generative']);

		$aibg_api_key 		= $_POST['aibg_api_key'];
	
		$aibg_prompt_key 	= $aibg_generative != 'false' ? 'prompt' : 'msg';
	
		$aibg_prompt 		= 'as a blogger, you are writing a blog about ' . $post_title . '. generate a ' . $this->aibg_convert_to_ordinal($aibg_data_index) . ' paragraph idea in less than 25 words';
	
		$aibg_api_url 		= 'https://askcybex.com/api/chat';
	
		$aibg_api_data 		= array(
			'api_key' 			=> $aibg_api_key,
			$aibg_prompt_key 	=> $aibg_prompt
		);
	
		$response 			= wp_remote_post($aibg_api_url, array(
			'method' 	=> 'POST',
			'body' 		=> $aibg_api_data,
			'timeout' 	=> 45,
			'headers' 	=> array()
		));
	
		if ( is_wp_error($response) ) 
		{
			$error_message = $response->get_error_message();

			echo wp_json_encode(array(
				'success' 	=> false,
				'error' 	=> $error_message
			));
		} 
		else 
		{
			$body 				= wp_remote_retrieve_body($response);

			$aibg_api_response 	= json_decode($body);
	
			if ( isset($aibg_api_response->response) ) 
			{
				$aibg_description = $aibg_api_response->response;
	
				echo wp_json_encode(array(
					'success' 			=> true,
					'aibg_description' 	=> $aibg_description
				));
			} 
			else 
			{
				echo wp_json_encode(array(
					'success' 	=> false,
					'error' 	=> isset($aibg_api_response->error) ? $aibg_api_response->error : 'Unknown error'
				));
			}
		}
		exit;
	}
	

	function aibg_generate_all_description() {

		if ( !isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'nonce') ) 
		{
			wp_send_json_error('Invalid nonce');
		}

		$aibg_desc_words_arr 	= $_POST['aibg_desc_words_arr'];

		$post_title 			= $_POST['post_title'];

		$aibg_generative 		= trim($_POST['aibg_generative']);

		$aibg_api_key 			= $_POST['aibg_api_key'];
	
		$aibg_prompt_key 		= $aibg_generative != 'false' ? 'prompt' : 'msg';
	
		$response 				= array();
	
		if ( !empty($aibg_desc_words_arr) ) 
		{
			$response['aibg_response_desc'] = '';
	
			foreach ( $aibg_desc_words_arr as $aibg_desc_words_arr_value ) {

				$aibg_api_url 	= 'https://askcybex.com/api/chat';
	
				$aibg_api_data 	= array(
					'api_key' 			=> $aibg_api_key,
					$aibg_prompt_key 	=> 'As a blogger, you are writing a blog about ' . $post_title . '. Generate a 200-300 words paragraph about ' . $aibg_desc_words_arr_value
				);
	
				$api_response 	= wp_remote_post($aibg_api_url, array(
					'method' 	=> 'POST',
					'body' 		=> $aibg_api_data,
					'timeout' 	=> 45,
					'headers' 	=> array()
				));
	
				if ( is_wp_error($api_response) ) 
				{
					$response['error'] 	= $api_response->get_error_message();

					$response['success'] = false;

					break;
				} 
				else 
				{
					$body 				= wp_remote_retrieve_body($api_response);

					$aibg_api_response 	= json_decode($body);
	
					if ( isset($aibg_api_response->response) ) 
					{
						$aibg_response_desc 	= $aibg_api_response->response;
	
						$response['aibg_response_desc'] .= '<div class="aibg_api_field">
							<textarea name="" rows="12" class="aibg_textarea_sections">' . $aibg_response_desc . '</textarea>
						</div>';
	
						$response['success'] = true;
					} 
					else 
					{
						$response['error'] 		= isset($aibg_api_response->error) ? $aibg_api_response->error : 'Unknown error';
						$response['success'] 	= false;
						break;
					}
				}
			}
		}
	
		echo wp_json_encode($response);
		exit;
	}
	

	function aibg_generate_header_image() {

		if ( !isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'nonce') ) 
		{
			wp_send_json_error('Invalid nonce');
		}

		$aibg_header_image_desc = $_POST['aibg_header_image_desc'];

		$aibg_generative 		= trim($_POST['aibg_generative']);

		$aibg_api_key 			= $_POST['aibg_api_key'];
	
		$aibg_prompt 			= 'assume you are a graphics designer ' . $aibg_header_image_desc;
	
		$aibg_api_url 			= 'https://askcybex.com/api/chat';
	
		$aibg_api_data 			= array(
			'api_key' 		=> $aibg_api_key,
			'image_prompt' 	=> $aibg_prompt,
			'quality' 		=> 'standard',
			'size' 			=> '1024x1024'
		);
	
		$api_response 			= wp_remote_post($aibg_api_url, array(
			'method' 	=> 'POST',
			'body' 		=> $aibg_api_data,
			'timeout' 	=> 60,
			'headers' 	=> array()
		));
	
		$response 				= array();
	
		if ( is_wp_error($api_response) ) 
		{
			$response['success'] 	= false;
			$response['error'] 		= $api_response->get_error_message();
		} 
		else 
		{
			$body 				= wp_remote_retrieve_body($api_response);
			$aibg_api_response 	= json_decode($body);
	
			if ( isset($aibg_api_response->response) ) 
			{
				$aibg_api_response_content 	= json_decode($aibg_api_response->response);

				$aibg_header_image 			= $aibg_api_response_content->URL;
	
				$response['success'] 		= true;

				$response['aibg_header_image'] = $aibg_header_image;
			} 
			else 
			{
				$response['success'] 	= false;
				$response['error'] 		= isset($aibg_api_response->error) ? $aibg_api_response->error : 'Unknown error';
			}
		}
		echo wp_json_encode($response);
		exit;
	}
	

	function generate_random_filename($extension = 'png') {

		$random_string 	= bin2hex(random_bytes(10));

		return $random_string . '.' . $extension;
	}

	function aibg_save_post_as_draft(){

		if ( !isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'nonce') ) 
		{
			wp_send_json_error('Invalid nonce');
		}

		$response = array();

		$post_featured_image = $_POST['post_featured_image'];
		$post_title = $_POST['post_title'];
		$post_content = $_POST['post_content'];
		$post_type = $_POST['post_type'];

		$post_data = array(
			'post_title'    => wp_strip_all_tags($post_title),
			'post_content'  => $post_content,
			'post_status'   => 'draft',
			'post_type' 	=> $post_type
		);
	
		$post_id = wp_insert_post($post_data);
	
		if ( $post_id ) 
		{
			if ( !empty( $post_featured_image ) ) 
			{
				require_once( ABSPATH . 'wp-admin/includes/file.php' );
	
				$temp_file = download_url( $post_featured_image );
	
				if( !is_wp_error( $temp_file ) ) 
				{
					$file_info 		= pathinfo($post_featured_image);
					$file_extension = isset($file_info['extension']) ? $file_info['extension'] : 'png'; // Default to png if no extension is found
		
					$new_file_name 	= $this->generate_random_filename($file_extension);
		
					$file = array(
						'name'     => $new_file_name,
						'type'     => mime_content_type( $temp_file ),
						'tmp_name' => $temp_file,
						'size'     => filesize( $temp_file ),
					);
					
					$sideload = wp_handle_sideload(
						$file,
						array(
							'test_form'   => false
						)
					);
		
					if( empty( $sideload[ 'error' ] ) ) {

						$attachment_id = wp_insert_attachment(
							array(
								'guid'           => $sideload[ 'url' ],
								'post_mime_type' => $sideload[ 'type' ],
								'post_title'     => basename( $sideload[ 'file' ] ),
								'post_content'   => '',
								'post_status'    => 'inherit',
							),
							$sideload[ 'file' ]
						);

						if( !is_wp_error( $attachment_id ) && $attachment_id ) 
						{
							require_once( ABSPATH . 'wp-admin/includes/image.php' );
				
							wp_update_attachment_metadata(
								$attachment_id,
								wp_generate_attachment_metadata( $attachment_id, $sideload[ 'file' ] )
							);
					
							set_post_thumbnail($post_id, $attachment_id);
						}
					}
				}
			}

			if ( $post_type == 'page' ) 
			{
				$aibg_view_link = add_query_arg(array(
					'page_id' => $post_id,
					'preview' => 'true',
					'_wpnonce' => wp_create_nonce('post_preview_' . $post_id)
				), home_url('/'));
			}
			else
			{
				$aibg_view_link = add_query_arg(array(
					'p' => $post_id,
					'preview' => 'true',
					'_wpnonce' => wp_create_nonce('post_preview_' . $post_id)
				), home_url('/'));
			}
			
			$response['success'] = true;

			$response['aibg_view_link'] = $aibg_view_link;
		}

		echo wp_json_encode( $response );
		exit;
	}

	function aibg_add_admin_menu(){
		
		add_menu_page(
			'Askcybex AI - Blog Generation',
			'Askcybex AI - Blog Generation',
			'manage_options',
			'askcybex-ai-blog-generation',
			array( $this, 'aibg_content_generator' ),
			'dashicons-media-default',
			6
		);
	}

	function aibg_content_generator(){

		include_once( AIBG_ADMIN_DIR . '/forms/aibg_content_generator.php' );

	}

	function aibg_convert_to_ordinal($number) {

		if (!is_numeric($number)) {
			return $number;
		}
	
		$last_two_digits 	= $number % 100;

		$last_digit 		= $number % 10;
	
		if ( $last_two_digits >= 11 && $last_two_digits <= 13 ) 
		{
			$suffix = 'th';
		} 
		else 
		{
			switch ($last_digit) {
				case 1:
					$suffix = 'st';
					break;
				case 2:
					$suffix = 'nd';
					break;
				case 3:
					$suffix = 'rd';
					break;
				default:
					$suffix = 'th';
			}
		}
	
		return $number . $suffix;
	}
	
	/**
	 * Adding Hooks
	 *
	 * @package Askcybex AI - Blog Generation
	 * @since 1.0.0
	 */
	function add_hooks(){
		
		add_action( 'wp_ajax_aibg_save_api_key', array( $this, 'aibg_save_api_key' ) );
		add_action( 'wp_ajax_nopriv_aibg_save_api_key', array( $this, 'aibg_save_api_key' ) );

		add_action( 'wp_ajax_aibg_generate_title', array( $this, 'aibg_generate_title' ) );
		add_action( 'wp_ajax_nopriv_aibg_generate_title', array( $this, 'aibg_generate_title' ) );

		add_action( 'wp_ajax_aibg_generate_single_description', array( $this, 'aibg_generate_single_description' ) );
		add_action( 'wp_ajax_nopriv_aibg_generate_single_description', array( $this, 'aibg_generate_single_description' ) );

		add_action( 'wp_ajax_aibg_generate_all_description', array( $this, 'aibg_generate_all_description' ) );
		add_action( 'wp_ajax_nopriv_aibg_generate_all_description', array( $this, 'aibg_generate_all_description' ) );
		
		add_action( 'wp_ajax_aibg_generate_header_image', array( $this, 'aibg_generate_header_image' ) );
		add_action( 'wp_ajax_nopriv_aibg_generate_header_image', array( $this, 'aibg_generate_header_image' ) );

		add_action( 'wp_ajax_aibg_save_post_as_draft', array( $this, 'aibg_save_post_as_draft' ) );
		add_action( 'wp_ajax_nopriv_aibg_save_post_as_draft', array( $this, 'aibg_save_post_as_draft' ) );

		add_action( 'admin_menu', array( $this, 'aibg_add_admin_menu' ) );
	}
}
?>